/*
 * timer.c
 *
 */

volatile int _step_main;
volatile int _speed_hz;

// Set step speed in Hz (1Hz - 50kHz)
void SetEngineSpeedHz(int hz){

  int new_ICR1;
  
  // Determine new values for prescaler and TOP
  // Prescaler so that ~ 1000 < TOP < 65536

  if(hz == 0){
    // Stop timer
    TCCR1B = (TCCR1B & ~(0x7 << CS10));
    
    // Update current speed
    _speed_hz = 0;

    return;
  }
  else if(hz <= 50){
    // Prescaler 256
    TCCR1B = (TCCR1B & ~(0x7 << CS10)) | (1 << CS12);
    new_ICR1 = F_CPU / (2 * 256 * hz);
  }
  else if(hz <= 200){
    // Prescaler 64
    TCCR1B = (TCCR1B & ~(0x7 << CS10)) | (1 << CS11) | (1 << CS10);
    new_ICR1 = F_CPU / (2 * 64 * hz);
  }
  else if(hz <= 2000){
    // Prescaler 8
    TCCR1B = (TCCR1B & ~(0x7 << CS10)) | (1 << CS11);
    new_ICR1 = F_CPU / (2 * 8 * hz);
  }
  else if(hz <= 50000){
    // Prescaler 1
    TCCR1B = (TCCR1B & ~(0x7 << CS10)) | (1 << CS10);
    new_ICR1 = F_CPU / (2 * 1 * hz);
  }
  
  // Wait for the timer to reset before updating ICR1
  while(TCNT1)
    ;

  // Set new TOP
  ICR1 = new_ICR1;

  // Set fill rate 50%
  OCR1A = OCR1B = ICR1 >> 1;

  // Update current speed
  _speed_hz = hz;
}

// Initialize Engine Timer
// Timer won't start before prescaler is set
void InitEngineTimer(){

  // Reset registers
  TCCR1A = TCCR1B = 0;

  // Set OC1A and OC1B as output
  DDRD |= (1 << PD5) | (1 << PD4);
  
  // Set PA5 (main engine) and 
  //     PA3 (sensor engine)
  // as output and enable sleep
  DDRA |= (1 << PA5) | (1 << PA3);
  PORTA &= ~((1 << PA5) | (1 << PA3));

  // Set PA7 and PA6 (main engine)
  // as output for direction control
  DDRA |= (1 << PA7) | (1 << PA6);

  // Phase and Frequency correct PWM 16-bit
  // TOP in ICR1
  TCCR1B |= (1 << WGM13);

  // Enable timer output on OC1A
  // Set when upcounting clear when downcounting
  TCCR1A |= (1 << COM1A1) | (1 << COM1A0) |
            (1 << COM1B1) | (1 << COM1B0);

}

// Enable main engine
void EnableMainEngine(){

  // Enable interrupts on capture (TOP)
  TIMSK1 |= (1 << ICIE1);

  // Disable sleep
  PORTA |= (1 << PA5);
  _delay_ms(10);

  // Initialize defaul engine parameters
  _step_main = 0;

  // Set engines to stop and hold
  SetEngineSpeedHz(0);

}

// Disable main engine
void DisableMainEngine(){

  // Set engines to stop and hold
  SetEngineSpeedHz(0);

  // Disable interrupts on capture (TOP)
  TIMSK1 &= ~(1 << ICIE1);

  // Enable sleep
  PORTA &= ~(1 << PA5);

}

// Count total number of steps taken
ISR(TIMER1_CAPT_vect){
  _step_main++;
}

void Move(){

  int i;
  char str[64];

  // Set direction on both engines as forward
  PORTA |= (1 << PA7) | (1 << PA6);

  EnableMainEngine();
  _delay_ms(2000);

  for(i = 0; i < 10; i++){
    SetEngineSpeedHz(i * 50);
    _delay_ms(10);
  }
 
  for(i = 0; i < 5; i++){
    _delay_ms(1000);
    sprintf(str, "S: %4d", _step_main);
    LCDPrintXY(str, 80, 0);
  }  

  for(i = 10 - 1; i >= 0; i--){
    SetEngineSpeedHz(i * 50);
    _delay_ms(10);
  }
  
  //_delay_ms(1000);

  DisableMainEngine();
  _delay_ms(2000);

}
